package com.ibm.able.examples.ablebean;

//====================================================================
//
// Licensed Materials - Property of IBM
//
// "Restricted Materials of IBM"
//
// Product: com.ibm.able.examples.ablebean
//
// (C) Copyright IBM Corp. 2000 All Rights Reserved.
//
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with 
// IBM Corp.
//
//                                                                        
//                             DISCLAIMER                                 
//                             ----------                                 
//                                                                        
// This material contains programming source code for your consideration. 
// These examples have not been thoroughly tested under all conditions.   
// IBM, therefore, cannot guarantee or imply reliability, serviceability, 
// performance or function of these programs.  All programs contained     
// herein are provided to you "AS IS".  THE IMPLIED WARRANTIES OF         
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY     
// DISCLAIMED.                                                            
//                                                                                                                                               
//====================================================================

//====================================================================
// Imports
//====================================================================
import java.awt.* ;
import java.io.* ;
import java.rmi.*;
import java.util.*;
import javax.swing.*;

import com.ibm.able.*;

/**
 *  This class is an example of a simple AbleBean created by extending 
 *  AbleObject.  It monitors specific files for any changes
 *  and also answers queries regarding file attributes. 
 *
 * @version  $Revision: 1.6 $, $Date: 2000/09/15 13:45:22 $
 *
 * @author   Joe Bigus
 */
 public class AbleFileWatcher extends AbleObject implements Serializable {

   //==================================================================
   //
   // Class Variables
   //
   //==================================================================

  /** 
   * This variable is used by Serializable interface.
   */
   static final long    serialVersionUID = 1999100100000000001L;
   
  /** 
   * This condition indicates the file is to be watched for changes.
   */
   static public final int FILE_MODIFIED  = 0;
  /** 
   * This condition indicates the file is to be watched for deletion.
   */
   static public final int FILE_DELETED   = 1;
  /** 
   * This condition indicates the file is to be watched for a size threshold.
   */
   static public final int FILE_THRESHOLD = 2;

  /** 
   * Take no action regardless of file condition.
   */
   static public final int ACTION_NONE    = 0;    
  /** 
   * Display a dialog when any condition is true.
   */
   static public final int ACTION_ALERT   = 1;    
  /** 
   * Start a process when any condition is true.
   */
   static public final int ACTION_EXECUTE = 2;  
  /** 
   * Send an event to a registered agent or bean when any condition is true.
   */
   static public final int ACTION_EVENT   = 3;    

   //==================================================================
   //
   // Instance Variables
   //
   //==================================================================
   
   private int    action           = ACTION_EVENT;    // default action is to send an event
   private String command          = "";    // command to execute
   private int    condition        = FILE_MODIFIED;  // type of file change
   private Dialog dialog           = null;  // dialog to display for alert
   private File   file             = null;  // no default file
   private String fileName         = null;  // no default file name
   private long   lastModified     = 0;     // last file modification
   private long   threshold        = Long.MAX_VALUE;    // file size threshold


  //==================================================================
  //
  // Constructors
  //
  //==================================================================
  /**
   * Construct an AbleFileWatcher with a name of FileWatcher.
   *
   * @return    AbleFileWatcher.
   */
   public AbleFileWatcher() throws RemoteException {
       this("FileWatcher");
   }

  /**
   * Construct an AbleFileWatcher with the provided bean name.
   *
   * @param     theBeanName A String containing the name of the object.
   * @return    AbleFileWatcher.
   */
   public AbleFileWatcher(String theBeanName) throws RemoteException {
       super(theBeanName);
       reset();
   }

  /**
   * Construct an AbleFileWatcher with the provided bean name
   * and on the specified file.
   *
   * @param     theBeanName A String containing the name of the object.
   * @param     theFileName A String containing the name of the file to watch.
   * @return    AbleFileWatcher.
   */
   public AbleFileWatcher(String theBeanName, String theFileName) throws RemoteException {
       super(theBeanName);
       setFileName(theFileName, false);
       reset();
   }

  //==================================================================
  //
  // Accessor methods for instance variables
  //
  //==================================================================
    
 /**
  * Set the type of action to take when the condition watched is true.
  * @param theAction An integer representing the action to take.
  * @see AbleFileWatcher#getAction
  */
  public void setAction(int theAction) throws RemoteException { 
    firePropertyChange("fileWatcherAction", new Integer(action), new Integer(theAction));
    action = theAction; 
  }  
 /**
  * Set the type of action to take when the condition watched is true.
  * @see AbleFileWatcher#ACTION_NONE
  * @see AbleFileWatcher#ACTION_ALERT
  * @see AbleFileWatcher#ACTION_EXECUTE
  * @see AbleFileWatcher#ACTION_EVENT
  * @return An integer representing the file condition.
  */
  public int getAction() { return action ; }

 /**
  * Set the command to execute when the condition is true and
  * the action selected is ACTION_EXECUTE.
  * @param theCommand A String containing the command to execute.
  */
  public void setCommand(String theCommand) throws RemoteException { 
    firePropertyChange("fileWatcherCommand", command, theCommand); 
    theCommand = command; 
  }
 /**
  * Get the command executed when the condition is true and
  * the action selected is ACTION_EXECUTE.
  * @return A String containing the command to execute.
  */
  public String getCommand() { return command; }

 /**
  * Set the file watch condition.  Generally this value is
  * expected to be calculated.  
  * @param theCondition An int representing the watch condition.
  * @see AbleFileWatcher#getCondition
  */
  public void setCondition(int theCondition) throws RemoteException { 
    firePropertyChange("fileWatcherCondition", new Integer(condition), new Integer(theCondition));
    condition = theCondition; 
  }
 /**
  * Get the file change condition.
  * @see AbleFileWatcher#FILE_DELETED
  * @see AbleFileWatcher#FILE_MODIFIED
  * @see AbleFileWatcher#FILE_THRESHOLD
  * @return An integer representing the file condition.
  */
  public int getCondition() { return condition ; }

 /**
  * Set the Dialog to display when the condition is true and
  * the action selected is ACTION_ALERT.
  * @param theDialog A Dialog to display.
  */
  public void setDialog(Dialog theDialog) throws RemoteException { 
    firePropertyChange("fileWatcherDialog", dialog, theDialog); 
    dialog = theDialog; 
  }
 /**
  * Get the Dialog to display when the condition is true and
  * the action selected is ACTION_ALERT.
  * @return A Dialog to display.
  */
  public Dialog getDialog() {return dialog;}

 /**
  * Set the file name and get a reference to the file.
  *
  * @param theName A String containing the path and name of the file to watch.
  */
  public void setFileName(String theName) throws RemoteException { 
    setFileName(theName, true);
  }
  
 /**
  * Set the file name and get a reference to the file. 
  *
  * @param theName A String containing the path and name of the file to watch.
  * @param sendChange A boolean indicating whether to fire a property change.
  */
  private void setFileName(String theName, boolean sendChange) throws RemoteException {
    if (sendChange) firePropertyChange("fileWatcherFileName", fileName, theName); 
    fileName = theName;
    file = new File(theName);
    lastModified = file.lastModified(); 
  }

 /**
  * Set the file name and get a reference to the file.
  */
  public String getFileName() { return fileName; }

 /**
  * Set the file size threshold condition.
  * The watch condition is true if the file size exceeds
  * this value.  
  * @param theThreshold A long containing the file size threshold.
  */
  public void setThreshold(long theThreshold) throws RemoteException { 
    firePropertyChange("fileWatcherThreshold", new Long(threshold), new Long(theThreshold));
    threshold = theThreshold; 
  }
 /**
  * Get the file size threshold.
  * @return A long representing the file size threshold.
  */
  public long getThreshold() { return threshold; }

  //==================================================================
  //
  // Methods to override AbleObject
  //
  //==================================================================
  /**
   * Initialize the bean to a known state supporting only timer events.
   *
   */
   public void init() throws RemoteException {
     
       // Enable timer processing, but not asynch events
       setTimerEventProcessingEnabled(true);
       setAbleEventProcessingEnabled(Able.ProcessingDisabled_PostingDisabled);

       // create/resume the  event queue thread if file specified
       if (file != null) {
         // no point in doing because we just start processing so it goes
         // straight to active/waiting
         // setState(AbleState.Initiated);
         startEnabledEventProcessing();    
       }
  }
  
  
  /**
   * Reset the bean to a known initialized state.
   *
   */
  public void reset() throws RemoteException {
       setSleepTime(5000);   // call processTimerEvent() every 5 seconds
      
       // set defaults for member variables
       action           = ACTION_ALERT; 
       command          = "";    
       condition        = FILE_THRESHOLD;  
       Dialog dialog    = null;  // dialog to display for alert
       if (file == null) lastModified = 0;
       else lastModified     = file.lastModified();     // last file modification
       threshold        = Long.MAX_VALUE;    // file size threshold

       init() ; 
  }
  
  
 /**
  *  Check the file being watched for the
  *  condition of interest.  If the condition occurred, perform the
  *  action specified.
  *
  * @exception An exception is thrown if an action fails.
  */
  public void process() throws RemoteException {
      if (checkCondition()) {
          performAction() ;
      }
  }

  /**
  *  The timer has expired, so check the file being watched for the
  *  condition of interest.  If the condition occurred, perform the
  *  action specified.
  *
  * @exception An exception is thrown if an action fails.
   */
   public void processTimerEvent() throws RemoteException {
      process() ; // just call process
   }


  //==================================================================
  //
  // Miscellaneous methods
  //
  //==================================================================


  /**
   *  Check to see if the condition monitored is true.
   *  @return boolean A boolean value which indicates whether the
   *                  desired condition is true.
   */
   public boolean checkCondition() {
       boolean truth = false ;
       if (file == null) return truth;
       switch (condition) {
         case FILE_MODIFIED:
            truth = changed() ;
            break ;
         case FILE_DELETED:      
            truth = !exists() ;  
            break ;
         case FILE_THRESHOLD:
            truth = file.length() > threshold;
            break ;
         default:
            truth = false;
            break;
       }
       return truth ;
   }

   /**
    *  Take the action that is specified by the user.
    */
    private void performAction() {
      switch (action) {
        case ACTION_NONE: 
            break;
        case ACTION_ALERT:
            System.out.println(name + ": Alert fired \n") ;
            if (dialog == null) {
                Object[] options = { "Continue", "Quit" }; 
                int ans = JOptionPane.showOptionDialog(null
                              , "Continue watching '" + fileName + "'?"
                              , "Continue?"
                              , JOptionPane.YES_NO_OPTION
                              , JOptionPane.QUESTION_MESSAGE
                              , null
                              , options
                              , options[1]);
                if (ans == JOptionPane.NO_OPTION) {
                    try {
                    this.quitEnabledEventProcessing();
                    } catch (RemoteException e) {
                        System.out.println("Error: cannot quit event processing.");
                    }
                }
            } else {
                dialog.setVisible(true);
            }
            break ;
        case ACTION_EXECUTE:
            System.out.println(name + ": Executing command \n") ;
            try {
                if (command.length() > 0)
                    AblePanelHelper.run(command);
            } catch (Exception err) {
                System.out.println("Error: EXEC failed, " + err.toString()) ;
                err.printStackTrace();
            }
            break ;
        case ACTION_EVENT:
            try {
                 notifyAbleEventListeners(new AbleEvent(this,"watcher"));   // signal interested observer
                 System.out.println(name + ": Event sent \n") ;
             } catch (RemoteException e) {
                System.out.println("Error: notifyAbleEventListeners failed, " + e.toString()) ;
                e.printStackTrace();
             } 
             break ;
         default:
             System.out.println("Invalid action specified.");
             break;
      }
    }


 /**
  *  Return a string for display in a list box.  This displays the
  *  object name and state information.
  *
  *  @return String containing the object name, file being watched, and condition.
  */
  public String getDisplayString() {
    String out = new String() ;
    out = name + " fileName='" + fileName + "' cond='" + condition +"'";
    return out ;
  }


  //==================================================================
  //
  // Methods for querying the file being watched
  //
  //==================================================================
  /**
   * Check to see if the file being watched changed.
   *
   * @return boolean A boolean indicating whether the file changed since the last call.
   */
   public boolean changed() { 
     if (file == null) {
        return false;
     } else {
        long lclLastModified = lastModified ;
        lastModified = file.lastModified() ;
        return !(lastModified == lclLastModified) ;
     }
   }
   
  /**
   * Check to see if the file being watched actually exists.
   *
   * @return boolean A boolean indicating whether the file exits.
   */
   public boolean exists() { 
    if (file == null) return false;
    else return file.exists() ; }
  
  /**
   * Check to see if the file being watched is a directory.
   *
   * @return boolean A boolean indicating whether the file watched is a directory.
   */
   public boolean isDirectory() { 
    if (file == null) return false;
    else return file.isDirectory(); }
   
  /**
   * Return the timestamp of the last file modification.
   *
   * @return long A long containing the timestamp of the last file modification.
   */
   public long lastModified() { 
     if (file == null) return 0l;
     else return file.lastModified(); }

  /**
   * Return the length of the file watched.
   *
   * @return long A long containing the number of bytes in the file watched.
   */
   public long length() { 
     if (file == null) return 0l;
     else return file.length(); 
   }  
  


    /**
     * Determine the copyright of this class.
     *
     * @return    String A String containing this class's copyright statement.
     *            <p>
     */
    public static String Copyright() {
      return Able.Copyright;
    }


}
