package com.ibm.able.examples.platform;


//====================================================================
// Imports
//====================================================================
import java.beans.*;
import java.io.*;
import java.rmi.*;
import java.util.*;

import com.ibm.able.*;
import com.ibm.able.agents.*;
import com.ibm.able.platform.*;
import com.ibm.able.platform.agents.*;


/**
 * This class is a command line test case (it has a main() method)
 * that creates a fuzzy ruleset, a fuzzy ruleset listener, wires the
 * two together, and then processes the rules. Lots of listener
 * messages should result on the java console.
 *
 */
public class ExampleFipaCode extends Object implements PropertyChangeListener, Serializable {

  public ExampleFipaCode() {
  }


  //==================================================================
  //
  // Main method
  //
  //==================================================================
  public static void main(String args[]) {
    try {
      ExampleFipaCode lclExampleFipaCode = new ExampleFipaCode();

      lclExampleFipaCode.runSamples();
    }
    catch(Exception theException) {
      System.out.println("++ExampleFipaCode: Caught exception in main: " + theException);
    }

    System.exit(0);
  }


  //==================================================================
  //
  // Methods specific to this class
  //
  //==================================================================
  public void runSamples() {
    try {

      //**************************************************************
      //
      // Working with the FIPA AMS
      //
      //**************************************************************

      //--------------------------------------------------------------
      // In this section, the following are performed:
      //
      // (1) The PLATFORM preferences file is examined to determine
      // where the FIPA AMS agent is supposed to be located.
      //
      // (2) RMI is used to lookup and get addressability to the FIPA
      // AMS agent at the location designated in (1).
      //
      // (3) Register with the AMS as a property change listener. This 
      // will tell us when agents register with or deregister from the 
      // AMS.
      //
      // (4) A FIPA "Agent Platform Description" is retrieved from the 
      // AMS. Nothing is done with this except to print it out.
      //
      // (5) Retrieve a list of all the agents currently registered
      // with the AMS, and print out the number of them.
      //
      //--------------------------------------------------------------

      // (1) Find out on which host the FIPA AMS agent is supposed to
      // reside. This information comes from the platform.preferences
      // file, which is user editable and must be set before running
      // this program.
      String lclAmsHost = Platform.Preferences.getAmsLocation();
      logTrace1("The FIPA AMS agent is supposed to be running on host <" + lclAmsHost + ">");

      // (2) Try to get addressability to the FIPA AMS agent from the
      // host determined in step (1).
      FipaAmsAgent lclAms = (FipaAmsAgent)Naming.lookup( Able.AgentName(Fipa.AmsAgentName, lclAmsHost) );
      logTrace1("Got addressability to the AMS running on host <" + lclAmsHost + ">");
      logTrace1("The AMS is: " + lclAms);
      logTrace2("  ");

      // (3) Register with the AMS to listen for agent registrations
      // and deregistrations.
      lclAms.addPropertyChangeListener(this);

      // (4) Use the AMS to retrieve a FIPA Agent Platform Description.
      FipaApDescription lclApd = lclAms.queryPlatformProfile();
      logTrace1("Retrieved the Agent Platform Description from the AMS. It follows: ");
      logTrace1(lclApd.toString());
      logTrace2("  ");

      // (5) Get a list of all the agents currently registered with
      // the AMS.
      Hashtable lclHash     = lclAms.getRegisteredAgents();
      int       lclHashSize = lclHash.size();
      logTrace1("The current number of agents registered with the AMS is " + lclHashSize);


      //--------------------------------------------------------------
      // In this section, the following are performed:
      //
      // (1) A FIPA AMS Agent Description is created, and that is
      // passed to the AMS where the new agent is created and
      // automatically registered.
      //
      // (2) Retrieve a list of all the agents currently registered
      // with the AMS, and print out the number of them. Assuming no
      // other activity at the AMS, the agent count should be 1
      // greater than before.
      //
      // (3) Dump all the currently registered agents to the console.
      //
      // (4) Search the AMS for the newly created agent, and, if
      // found, print it out using different references.
      //
      // (5) Add a property change listener to the agent and then run
      // some methods on the newly created agent.
      //
      //--------------------------------------------------------------

      // (1) Create a FIPA AMS Agent Description of an
      // AbleDefaultAgent and then try to create the described agent
      // and register it with the AMS:
      String lclWhere = "localhost";
      String lclAddr  = Able.AgentAddr(lclWhere);
      String lclName  = Able.AgentName("ExampleFipaAgent_One", lclWhere);

      FipaAmsAgentDescription lclAmsAd = new FipaAmsAgentDescription(lclName, lclAddr);
      lclAmsAd.setApState(Fipa.ApStateActiveId);
      lclAmsAd.setDelegateAgentName(Able.AgentName("DummyAgentDelegator", lclWhere));
      lclAmsAd.setOwnership("Snazzy-eCorporation");
      lclAmsAd.setSignature("Sig9F3Deg7gGzIl");
      lclAmsAd.setClassName("com.ibm.able.agents.AbleDefaultAgent");
      lclAmsAd.setHostName (lclWhere);

      // Use the AMS to create the new intelligent agent.
      // Notice that the createAgent() method returns a reference to
      // our newly created agent!
      logTrace1("Using AMS to create a new AbleDefaultAgent on " + lclWhere + "...");
      AbleAgent lclAgent1a = (AbleAgent)lclAms.createAgent(lclAmsAd);
      lclAgent1a.setName("ExampleFipaAgent_One");
      lclAgent1a.init();
      logTrace1("  Creation complete. New agent is " + lclAgent1a);
      logTrace2("  ");

      // (2) Get a list of all the agents currently registered with
      // the AMS.
      lclHash     = lclAms.getRegisteredAgents();
      lclHashSize = lclHash.size();
      logTrace1("The number of agents registered with the AMS is now " + lclHashSize);

      // (3) Dump all the currently registered agents to the console.
      logTrace1("Printing AMS agent descriptions of all AMS registered agents... ");
      Enumeration lclEnum = lclHash.elements();
      while (lclEnum.hasMoreElements()) {
        FipaAmsAgentDescription tmpAmsAd = (FipaAmsAgentDescription)lclEnum.nextElement();
        logTrace1("Registered agent AMS description... " + tmpAmsAd);
      }
      logTrace2("  ");

      // (4) Search the AMS for the newly created agent (using our
      // original AMS Agent Description), and, if found, print it out
      // using different references.
      logTrace1("Searching AMS on " + lclAmsHost + " for newly created agent...");
      FipaAmsAgentDescription lclAmsAd1 = lclAms.searchAgent(lclAmsAd);
      logTrace1("  Agent found. Returned description is: " + lclAmsAd1);
      logTrace2("  ");

      // Extract a reference to the agent from the agent description.
      AbleAgent lclAgent1b = (AbleAgent)lclAmsAd1.getReference();
      logTrace1("AMS Agent Description references AbleAgent  " + lclAgent1b);
      logTrace2("  ");

      // This particular agent is also an AbleEventListener, so pull 
      // out the reference to the agent from the description again,
      // but this time cast it as an AbleEventListener.
      AbleEventListener lclAgent1c = (AbleEventListener)(lclAmsAd1.getReference());
      logTrace1("The same agent as an AbleEventListener " + lclAgent1c);
      logTrace2("  ");


      // (5) Add a property change listener to the agent and then run
      // some methods on the newly created agent.
      logTrace1("Adding property change listener to agent via reference 2.");
      lclAgent1b.addPropertyChangeListener(this);

      String lclComment = lclAgent1b.getComment();
      logTrace1("Agent's original comment is: " + lclComment);

      logTrace1("Generating a property change event (setComment) via reference 1.");
      lclAgent1a.setComment("CHANGED COMMENT 1");

      logTrace1("Generating a property change event (setComment) via reference 2.");
      lclAgent1b.setComment("changed_comment-2");

      lclComment = lclAgent1a.getComment();
      logTrace1("Agent's comment is now: " + lclComment);

      logTrace1("Removing property change listener via reference 2.");
      lclAgent1b.removePropertyChangeListener(this);
      logTrace2("  ");


      //--------------------------------------------------------------
      // In this section, the following are performed:
      //
      // (1) Another AbleDefaultAgent is created, but this agent
      // created locally in the program, as opposed to using the AMS
      // to create an agent externally on this computer or some other
      // computer.
      //
      // (2) An AbleEvent is sent to the new agent. The agent should 
      // handle the event asyncronously.
      //
      // (3) Unbind from RMI.
      //
      //--------------------------------------------------------------

      // (1) Create a new agent locally here in this program and make
      // it available through RMI. The agent is not registered with
      // the AMS.
      AbleDefaultAgent lclAgent2 = new AbleDefaultAgent("aNewLocalAgent", "This agent is local to the program.");
      lclAgent2.init();
      lclAgent2.rmiRebind();

      // (2) Create an AbleEvent and send it to this new agent.
      logTrace1("Creating an Able event.");
      AbleEvent lclEvent = new AbleEvent((AbleAgent)lclAgent2, null);
      logTrace1("The event is... " + lclEvent);
      logTrace2("  ");

      logTrace1("Sending the Able event to the local agent...");
      ((AbleEventListener)lclAgent1a).handleAbleEvent(lclEvent);
      logTrace1("  Event sent.");
      logTrace2("  ");

      // (3) Unbind from RMI.
      lclAgent2.rmiUnbind();
      lclAgent2 = null;


      //--------------------------------------------------------------
      // Finally, we remove our property change listener from the AMS
      // because we no longer care about registrations or
      // deregistrations there.
      //--------------------------------------------------------------
      lclAms.removePropertyChangeListener(this);



      //**************************************************************
      //
      // Working with the FIPA Directory Facilitator (DF)
      //
      //**************************************************************

      //--------------------------------------------------------------
      // In this section, the following are performed:
      //
      // (1) The PLATFORM preferences file is examined to determine
      // where the FIPA DF agent is supposed to be located.
      //
      // (2) RMI is used to lookup and get addressability to the FIPA
      // DF agent at the location designated in (1).
      //
      // (3) Register with the DF as a property change listener. This 
      // will tell us when agents register with or deregister from the 
      // DF.
      //
      // (4) Retrieve a list of all the agents currently registered
      // with the DF, and print out their DF descriptions.
      //
      // (5) Register the agent created in the AMS section above with
      // the DF.
      //
      // (6) Search the DF for our newly registered agent.
      //
      // (7) Deregister our newly registered agent.
      //
      //--------------------------------------------------------------

      // (1) Find out on which host the FIPA DF agent is supposed to
      // reside. This information comes from the platform.preferences
      // file, which is user editable and must be set before running
      // this program.
      logTrace2("  ");
      String lclDfHost = Platform.Preferences.getDfLocation();
      logTrace1("The FIPA DF agent is supposed to be running on host <" + lclDfHost + ">");

      // (2) Try to get addressability to the FIPA DF agent from the
      // host determined in step (1).
      // Of course, we could also ask the AMS for the DF.
      FipaDfAgent lclDf = (FipaDfAgent)Naming.lookup( Able.AgentName(Fipa.DfAgentName, lclDfHost) );
      logTrace1("Got addressability to the DF running on host <" + lclDfHost + ">");
      logTrace1("The DF is: " + lclDf);
      logTrace2("  ");

      // (3) Register with the DF to listen for agent registrations
      // and deregistrations.
      lclDf.addPropertyChangeListener(this);

      // (4) Get a list of all the agents currently registered with
      // the DF.
      logTrace1("Printing DF agent descriptions of all DF registered agents... ");
      Hashtable   lclHashDf = lclDf.getRegisteredAgents();
      Enumeration lclEnumDf = lclHashDf.elements();
      while (lclEnumDf.hasMoreElements()) {
        FipaDfAgentDescription tmpDfAd = (FipaDfAgentDescription)lclEnumDf.nextElement();
        logTrace1("Registered agent DF description... " + tmpDfAd);
      }
      logTrace2("  ");

      // (5) Register the agent created in the AMS section above with
      // the DF.

      // This will be the set of services supported by our agent.
      Vector lclServices = new Vector();

      // Create a FIPA Service Description and add it to the set of
      // services supported by our agent.
      FipaServiceDescription lclSd1 = new FipaServiceDescription();
      lclSd1.setServiceName("OurSuperDuper-agent-service");
      lclSd1.setServiceType("OurSuperDuper-service-type");
      lclSd1.setServiceOntology("OurSuperDuper-agent-management-onotology");

      lclServices.addElement(lclSd1);

      // Create a FIPA Service Description and add it to the set of
      // services supported by our agent.
      FipaServiceDescription lclSd2 = new FipaServiceDescription();
      lclSd1.setServiceName("YetAnother-agent-service");
      lclSd1.setServiceType("YetAnother-service-type");
      lclSd1.setServiceOntology("YetAnother-agent-management-onotology");

      lclServices.addElement(lclSd2);

      // Create a DF description of our agent
      FipaDfAgentDescription lclDfAd = new FipaDfAgentDescription();
      lclDfAd.setAgentName(lclName);
      lclDfAd.setAddress  (lclAddr);
      lclDfAd.setServices (lclServices);
      lclDfAd.setType     (Fipa.StFipaAgent);
      lclDfAd.setInteractionProtocols("fipa-request");
      lclDfAd.setOntology ("fipa-agent-management");
      lclDfAd.setOwnership(Platform.Owner);
      lclDfAd.setDfState  (Fipa.DfStateActiveId);
      lclDfAd.setReference(lclAgent1a);  // This is the agent created in the AMS section above.

      // Now register our agent with the DF.
      logTrace1("Registering agent <" + lclName + "> with the DF.");
      lclDf.register(lclDfAd);
      logTrace1("Registration complete.");

      // (6) Search the DF for our newly registered agent.
      logTrace1("Searching the DF for agent <" + lclName + ">");
      Vector lclDfSrch = lclDf.search(lclDfAd);
      logTrace1("DF search result: " + lclDfSrch);
      logTrace2("  ");

      // (7) Deregister our newly registered agent.
      logTrace1("Deregistering agent <" + lclName + "> from the DF.");
      FipaDfAgentDescription lclDfAd2 = lclDf.deregister(lclDfAd);
      logTrace1("Deregister result <" + lclDfAd2 + ">.");


      //--------------------------------------------------------------
      // Finally, we remove our property change listener from the DF
      // because we no longer care about registrations or
      // deregistrations there.
      //--------------------------------------------------------------
      lclDf.removePropertyChangeListener(this);



      //**************************************************************
      //
      // Working with the FIPA Agent Communication Channel (ACC)
      //
      //**************************************************************

      //--------------------------------------------------------------
      // In this section, the following are performed:
      //
      // (1) The PLATFORM preferences file is examined to determine
      // where the FIPA ACC agent is supposed to be located.
      //
      // (2) RMI is used to lookup and get addressability to the FIPA
      // ACC agent at the location designated in (1).
      //
      // (3) Use the AMS to create another agent, and then use the ACC 
      // to send both of our AMS-registered agents a FIPA message.
      //
      // (4) Create yet another agent, this time a local one, and send 
      // it a couple of FIPA ACL messages.
      //
      //--------------------------------------------------------------

      // (1) Find out on which host the FIPA ACC agent is supposed to
      // reside. This information comes from the platform.preferences
      // file, which is user editable and must be set before running
      // this program.
      logTrace2("  ");
      String lclAccHost = Platform.Preferences.getAccLocation();
      logTrace1("The FIPA ACC agent is supposed to be running on host <" + lclAccHost + ">");

      // (2) Try to get addressability to the FIPA ACC agent from the
      // host determined in step (1).
      // Of course, we could also ask the AMS for the ACC.
      FipaAccAgent lclAcc = (FipaAccAgent)Naming.lookup( Able.AgentName(Fipa.AccAgentName, lclAccHost) );
      logTrace1("Got addressability to the ACC running on host <" + lclAccHost + ">");
      logTrace1("The ACC is: " + lclAcc);
      logTrace2("  ");

      // (3) Use the AMS to create another agent, and then use the ACC 
      // to send both of our AMS-registered agents a FIPA message.

      // Create an agent description of another AbleDefaultAgent
      // and then ask the AMS to create this agent for us. The AMS
      // will auto-register the new agent.
      String lclWhere2 = "localhost";
      String lclAddr2  = Able.AgentAddr(lclWhere2);
      String lclName2  = Able.AgentName("ExampleFipaAgent_TWO", lclWhere2);

      FipaAmsAgentDescription lclAmsAd2 = new FipaAmsAgentDescription(lclName2, lclAddr2);
      lclAmsAd2.setApState(Fipa.ApStateActiveId);
      lclAmsAd2.setDelegateAgentName(Able.AgentName("DummyAgentDelegator_TWO", lclWhere2));
      lclAmsAd2.setOwnership("AnotherSnazzy-eCorporation");
      lclAmsAd2.setSignature("Sig29F3Deg7gGzIl+TWO");
      lclAmsAd2.setClassName("com.ibm.able.agents.AbleDefaultAgent");
      lclAmsAd2.setHostName (lclWhere2);

      // Ask the AMS to create the new agent
      logTrace1("Using AMS to create a another AbleDefaultAgent, this time on " + lclWhere2 + "...");
      AbleAgent lclAgent3 = (AbleAgent)lclAms.createAgent(lclAmsAd2);
      lclAgent3.setName("ExampleFipaAgent_TWO");
      lclAgent3.init();
      logTrace1("  Creation complete. New agent is " + lclAgent3);
      logTrace2("  ");

      // Tweak the new agent's comment.
      lclAgent3.setComment("This comment changed by ExampleFipaCode.");

      // Set up a list of agents to which our FIPA message will be
      // routed.
      Vector lclRcvrs = new Vector();
      lclRcvrs.addElement(lclName );
      lclRcvrs.addElement(lclName2);

      // Create the content of our FIPA message
      AbleEvent lclEvnt = new AbleEvent(this            // source
                                        , null          // arg
                                        , "getComment"  // method
                                        );

      // Create the actual FIPA ACL message and add our message
      // content and list of receivers to the message.
      FipaAclMessage lclMessage = new FipaAclMessage(Fipa.MtRequestId);
      lclMessage.setContent(lclEvnt);
      lclMessage.setReceiver(lclRcvrs);
      logTrace1("Created FIPA ACL message: " + lclMessage);
      logTrace2("  ");

      // Now, the real crux of the matter: ask the ACC to forward the
      // message to the identified receivers.
      logTrace1("Using FIPA ACC agent to forward message.");
      lclAcc.forward(lclMessage);
      logTrace1("  Done.");
      logTrace2("  ");


      // (4) Create yet another agent, this time a local one, and send 
      // it a couple of FIPA ACL messages.

      //--------------------------------------------------------------
      // NOTE!
      //
      // The agents created so far are AbleDefaultAgents. They can
      // communicate with one another by several means:
      //   (1) They can directly call each other's methods, even
      //   across a network, as we have seen.
      //   (2) They can send each other AbleEvent objects by
      //   directly calling the appropriate synchronous and
      //   asynchronous "handle" methods of each agent.
      //   (3) They can use a FIPA ACC agent to accomplish No. 2
      //   above.
      // The AbleEvent object allows any public method of any agent
      // to be indirectly invoked, passing the method any arbitrary
      // object. In the example immediately above, the getComment()
      // method of lclAgent3 is eventually invoked. But this is not
      // the FIPA way.
      //
      // In the next example, we create a FIPA-compliant agent that
      // understands all the true, niggly, little FIPA messages that
      // can be routed through an ACC. Agents derived from
      // FipaAgentDefaultAgent are easier to manage in a FIPA world
      // than AbleDefaultAgents, because agents derived from
      // FipaAgentDefaultAgent have AMS and DF descriptions objects
      // built into them which they manage themselves internally. They
      // also can self-register with any AMS and DF agents and respond
      // to regular FIPA ACL messages, as we are about to see.
      //--------------------------------------------------------------

      // Create a FIPA-compliant agent;
      // it self-registers with the AMS and DF when it is created.
      SimpleFipaAgent lclFipaAgent = new SimpleFipaAgent("SimpleFipaAgent_ONE", "This agent derived from FipaAgentDefaultAgent");
      lclFipaAgent.init();

      // Create a FIPA ACL Message and use the ACC to send it to
      // the test FIPA agent.
      Vector lclRcvrs2 = new Vector();
      lclRcvrs2.addElement(lclFipaAgent.getAgentName());

      FipaAclMessage lclMessage2 = new FipaAclMessage(Fipa.MtRequestId);
      lclMessage2.setContent("Any arbitrary object! Such as this String object.");
      lclMessage2.setReceiver(lclRcvrs2);

      logTrace1("Forwarding msg via ACC " + lclMessage2);
      lclAcc.forward(lclMessage2);
      logTrace1("  Done.");

      // And here is the special case again: the message content is an 
      // AbleEvent object. The event object specifies which method
      // to run.
      AbleEvent lclEvnt2 = new AbleEvent(this            // source
                                         , null          // arg
                                         , "quitAgent"   // method
                                         );

      FipaAclMessage lclMessage3 = new FipaAclMessage(Fipa.MtRequestId);
      lclMessage3.setContent(lclEvnt2);
      lclMessage3.setReceiver(lclRcvrs2);

      logTrace1("Forwarding QUIT msg via ACC " + lclMessage3);
      lclAcc.forward(lclMessage3);
      logTrace1("  Done.");


      // --------------------------------------------------------
      // Cleanup
      // --------------------------------------------------------
      logTrace1("  ");
      logTrace1("Deregistering generated agents from the AMS.");
      lclAms.deregisterAgent(lclAmsAd);
      lclAms.deregisterAgent(lclAmsAd2);
      logTrace1("  Done.");

      lclHash = lclAms.getRegisteredAgents();
      logTrace1("  Number of agents still registered at AMS: " + lclHash.size());

    }
    catch(Exception theException) {
      logTrace1("Caught exception: " + theException);
    }
  }


  public void propertyChange(PropertyChangeEvent theEvent) {
    logTrace1("Caught property change event. ");
    logTrace1("++ PropertyName... " + theEvent.getPropertyName());
    logTrace1("++ OldValue....... " + theEvent.getOldValue());
    logTrace1("++ NewValue....... " + theEvent.getNewValue());
    logTrace1("++ Complete event: " + theEvent);
  }


  /**
   * Log a trace message.
   *
   * @param     theMessage
   *            A String containing text to be logged to the Java console.
   *            <p>
   */
  private void logTrace1(String theMessage) {
    System.out.println("++ExampleFipaCode: " + theMessage);
  }


  /**
   * Log a trace message.
   *
   * @param     theMessage
   *            A String containing text to be logged to the Java console.
   *            <p>
   */
  private void logTrace2(String theMessage) {
    System.out.println("                  " + theMessage);
  }

}
