package com.ibm.able.examples.platform;


//====================================================================
// Imports
//====================================================================
import java.io.*;
import java.net.*;
import java.rmi.*;
import java.util.*;

import com.ibm.able.*;
import com.ibm.able.platform.*;
import com.ibm.able.platform.agents.*;


/**
 * This class provides a simple implementation of the FipaAgent interface. 
 *
 * <p>This agent does nothing really, but when created it will seek
 * out and register itself with the AMS and DF agents, and when told
 * to "quit" it will deregister from those agents. It can also accept
 * FIPA "request" messages, but simply prints a statement that it got
 * one. 
 *
 * @version  1.0.0; change log:
 *
 * <pre>
 * Version   Description
 * --------  -----------
 * 1.0.0     Initial release.
 *
 * </pre>
 *
 * @author   Don Schlosnagle
 */
public class SimpleFipaAgent extends FipaAgentDefaultAgent implements Serializable {

  //==================================================================
  //
  // Class Variables
  //
  //==================================================================


  //==================================================================
  //
  // Instance Variables
  //
  //==================================================================


  //==================================================================
  //
  // Constructors
  //
  //==================================================================
  /**
   * Create a new SimpleFipaAgent.
   * This method that takes no parameters allows this class to be used
   * by BeanBox-like construction tools. It simply sets the name of the
   * agent to a system generated name.
   *
   * @exception RemoteException
   *            On any error.
   */
  public SimpleFipaAgent() throws RemoteException {
    // Initialise my parent
    super();
  }


  /**
   * Create a new FipaAgent, specify it's name, and give it a comment.
   *
   * @param     theName
   *            A string containing the name of this new agent.
   *            <p>
   * @param     theComment
   *            A string containing a comment for this new agent.
   *            <p>
   * @exception RemoteException
   *            On any error.
   */
  public SimpleFipaAgent(String theName, String theComment) throws RemoteException {
    // Initialise my parent
    super(theName, theComment);
  }


  //==================================================================
  //
  // Methods from the FipaAgent interface.
  //
  //==================================================================

  //==================================================================
  // 1. Base FIPA agent control methods
  //==================================================================
  // Inherit all except:
  public void quitAgent() throws RemoteException {
    finalTakedown();
  }


  //==================================================================
  // 2. Methods to handle FIPA messages
  //==================================================================
  // Inherit all except:
  public void msgRequest(FipaAclMessage theAclMessage) throws RemoteException {
    debugTrace("Agent method <msgRequest(FipaAclMessage)> called.");
    debugTrace("  The argument is the following Acl Message object: ");
    debugTrace(theAclMessage.toString());
  }


  //==================================================================
  //
  // Methods from the AbleBean interface.
  //
  //==================================================================
  /**
   * Initialize and configure the agent.
   *
   * <p> In this implementation the bean's state changes from
   * AbleState.Uninitiated to AbleState.Initiated; timer event
   * processing is disabled; both Able event posting and processing
   * are enabled; and the asynchronous thread is started.
   *
   */
  public void init() throws RemoteException {
    super.init();

    setDataFlowEnabled(false);
    setTimerEventProcessingEnabled(false);
    setAbleEventProcessingEnabled(Able.ProcessingEnabled_PostingEnabled);
    startEnabledEventProcessing();

    // I'm now active!
    debugTrace("init(): Active!");

    // Register with RMI
    rmiRebind();
    debugTrace("init(): Agent made distributed.");

    // Fill in the AMS agent description of myself
    // and try to register myself with the AMS.
    // Note that "myAmsAd" is provided by the FipaAgentDefaultAgent class.
    myAmsAd.setAgentName(this.getAgentName ());
    myAmsAd.setAddress  (this.getAgentAddr ());
    myAmsAd.setApState  (this.getState());
    myAmsAd.setOwnership("MeMyselfAndI");
    myAmsAd.setSignature("MySig");
    myAmsAd.setClassName(clsNm());
    myAmsAd.setHostName (this.getAgentHost ());
    myAmsAd.setReference(this);

    debugTrace("init(): Registering with AMS.");
    registerWithAms();
    debugTrace("init(): Registration complete.");

    // Fill in the DF agent description of myself
    // and try to register myself with the DF.
    // Note that "myDfServices" and "myDfAd" are provided by the FipaAgentSupport class.
    FipaServiceDescription lclSd1 = new FipaServiceDescription();
    lclSd1.setServiceName(Fipa.StFipaAgent);
    lclSd1.setServiceType(Fipa.StFipaAgent);
    lclSd1.setServiceOntology("able-platform-example");

    myDfServices.addElement(lclSd1);

    myDfAd.setAgentName(getAgentName());
    myDfAd.setAddress  (getAgentAddr());
    myDfAd.setServices (myDfServices);
    myDfAd.setType     (Fipa.StFipaAgent);
    myDfAd.setInteractionProtocols("fipa-request");
    myDfAd.setOntology ("fipa-agent-management");
    myDfAd.setOwnership("MeMyselfAndI");
    myDfAd.setDfState  (Fipa.DfStateActiveId);
    myDfAd.setReference(this);

    debugTrace("init(): Registering with DF.");
    registerWithDf();
    debugTrace("init(): Registration complete.");

    debugTrace("init(): Initialisation complete.");
  }


  //==================================================================
  //
  // Methods overridden from the AbleEventListener interface (via the
  // AbleObject's AbleBean interface).
  //
  //==================================================================
  /**
   * Handle an Able event.
   *
   * @param     theAbleEvent
   *            The event to handle.
   *            <p>
   */
  public void handleAbleEvent(AbleEvent theAbleEvent) throws RemoteException {
    debugTrace("SimpleFipaAgent in handleAbleEvent(). Calling super....");
    super.handleAbleEvent(theAbleEvent); 
  }


  //==================================================================
  //
  // Methods specific to this class.
  //
  //==================================================================
  private void finalTakedown() throws RemoteException {
    debugTrace("QUIT: Agent method <quitAgent()> called.");

    // Stop the agent's thread.
    debugTrace("QUIT: Stopping asynch thread..");
    quitEnabledEventProcessing();
    debugTrace("QUIT: Complete.");

    // Unbind from RMI.
    debugTrace("QUIT: Unbinding from RMI.");
    rmiUnbind();
    debugTrace("QUIT: Complete.");

    // Deregister from the DF.
    debugTrace("QUIT: Deregistering from DF.");
    deregisterWithDf ();
    debugTrace("QUIT: Complete.");

    // Deregister from the AMS.
    debugTrace("QUIT: Deregistering from AMS.");
    deregisterWithAms();
    debugTrace("QUIT: Complete.");

    debugTrace("QUIT: QUIT complete.");
  }


  //==================================================================
  //
  // MAIN Method
  //
  //==================================================================
  public static void main(String args[]) throws RemoteException {
    System.setSecurityManager(new RMISecurityManager());

    try {
      SimpleFipaAgent lclFipaAgent = new SimpleFipaAgent();
      lclFipaAgent.init();
    }
    catch (Exception theException) {
      throw new RemoteException(theException.toString());
    }
  }


  //==================================================================
  //
  // Additional Methods.
  //
  //==================================================================
  /**
   * Retrieve a string describing (the contents of) the object.
   *
   * @return    A String containing the current contents of the object.
   */
  public String toString() {
    String lclPrf = "";
    String lclStr = "";
    String lclCls = (this.getClass()).getName();

    for (int i=0; i<Able.IndentLevel; i++) {
      lclPrf = lclPrf + " ";
    }

    Able.IndentLevel += 2;

    lclStr = lclStr + super.toString() + "\n";

    lclStr = lclStr + lclPrf +            lclCls + "---------------------------\n";
    lclStr = lclStr + lclPrf + "  Name    : \t" + name                      + "\n";
    lclStr = lclStr + lclPrf + "  Comment : \t" + comment                   + "\n";
    lclStr = lclStr + lclPrf + "--End " + lclCls + "---------------------------\n";

    Able.IndentLevel -= 2;

    return lclStr;
  }


  /**
   * Determine the copyright of this class.
   *
   * @return    A String containing this class's copyright statement.
   */
  public static String Copyright() {
    return Able.Copyright;
  }

}
