package com.ibm.able.examples.fuzzy;


//====================================================================
// Imports
//====================================================================
import java.io.*;
import java.lang.reflect.*;
import java.rmi.*;

import com.ibm.able.*;
import com.ibm.able.beans.*;
import com.ibm.able.beans.fuzzy.*;


/**
 * This class is a command line test case (it has a main() method)
 * that creates a fuzzy ruleset, a fuzzy ruleset listener, wires the
 * two together, and then processes the rules. Lots of listener
 * messages should result on the java console.
 *
 */
public class SampleSensorEffector extends AbleObject {

  //==================================================================
  //
  // Constructors
  //
  //==================================================================
  public SampleSensorEffector() throws RemoteException {
    this("SampleSensorEffector");
  }

  public SampleSensorEffector(String theName) throws RemoteException {
    super(theName);
    init();
  }


  public void process() throws RemoteException {
    runSample();
  }


  //==================================================================
  //
  // Methods specific to this class
  //
  //==================================================================
  public void runSample() {

    try {
      //--------------------------------------------------------------
      // Create a ruleset for testing
      //--------------------------------------------------------------
      FsTrace   lclTracerB = new FsTrace();  // Used by fuzzy base code
      FsTrace   lclTracerI = new FsTrace();  // Used by fuzzy inferencing
      FsTrace   lclTracerP = new FsTrace();  // Used by fuzzy parser
      FsRuleSet lclRuleSet = new FsRuleSet(lclTracerB, lclTracerI);

      lclTracerB.setActive(false); lclTracerB.setBatch(true );
      lclTracerI.setActive(false); lclTracerI.setBatch(true );
      lclTracerP.setActive(false); lclTracerP.setBatch(true );

      //--------------------------------------------------------------
      // Create a fuzzy ruleset listener and add it to the ruleset.
      // This is purposely done BEFORE the ruleset is instantiated.
      //--------------------------------------------------------------
      FsListener lclFuzzyListener  = new FsListener();
      lclRuleSet.addFuzzyChangeListener(lclFuzzyListener);

      //--------------------------------------------------------------
      // Instantiate the ruleset by reading a source rule file.
      // This is purposely done AFTER the listener is added above.
      //--------------------------------------------------------------
      String lclPath = Able.ProductDirectory+"examples"+File.separator+"fuzzy"+File.separator;
      lclRuleSet.instantiateFrom(lclPath + "SampleSensorEffector.frs", lclTracerP, false);

      //--------------------------------------------------------------
      // Create an input buffer with sample data and add the input
      // buffer to the ruleset.
      //--------------------------------------------------------------
      String[] lclInpBuffer = new String[ ] {"true"/*vB1*/, "9.9"/*vN1*/, "Ghi"/*vS1*/, "88.88"/*vC1*/};

      lclRuleSet.setInputBuffer(lclInpBuffer);

      //--------------------------------------------------------------
      // Create the sensors & effectors and add them to the ruleset.
      //--------------------------------------------------------------
      Class        lclClass    = this.getClass();
      Method       lclSenMethod= lclClass.getMethod("tstLockMethod", new Class[] {Object.class});
      Method       lclEffMethod= lclClass.getMethod("setLockMethod", new Class[] {Object.class});
      AbleSensor   lclSensor   = new AbleSensor  ("tstLock", this, lclSenMethod);
      AbleEffector lclEffector = new AbleEffector("setLock", this, lclEffMethod);

      lclRuleSet.addSensor  (lclSensor);
      lclRuleSet.addEffector(lclEffector);

      //--------------------------------------------------------------
      // Set up the input buffer
      //--------------------------------------------------------------
      ; // Nothing to do -- the buffer is initialized where it is declared.

      //--------------------------------------------------------------
      // Process the fuzzy ruleset.
      //--------------------------------------------------------------
      lclRuleSet.process();

      //--------------------------------------------------------------
      // Check the output buffer
      //--------------------------------------------------------------
      String[] lclOutBuffer = (String[])lclRuleSet.getOutputBuffer();

      for (int i=0; i<lclOutBuffer.length; i++) {
        System.out.println("!! Output element[" + i + "]: <" + lclOutBuffer[i] + ">");
      }
    }
    catch(Exception theException) {
      System.out.println("!! Caught nasty exception: " + theException);
    }
  }


  /**
   * The "sensor" named "tstLock".
   *
   * <p>This method simply returns a new copy of the argument.
   *
   */
  public Object tstLockMethod(Object theTstObject) {
    System.out.println("!! In SENSOR. Object is: " + theTstObject.getClass().getName() + ": <" + theTstObject.toString() + ">");
    return performSensorEffector(theTstObject);
  }


  /**
   * The "effector" named "setLock".
   *
   * <p>This method simply returns a new copy of the argument.
   *
   */
  public Object setLockMethod(Object theTstObject) {
    System.out.println("!! In EFFECTOR. Object is: " + theTstObject.getClass().getName() + ": <" + theTstObject.toString() + ">");
    return performSensorEffector(theTstObject);
  }


  /**
   * A helper routine used by the sensor and effector methods.
   *
   * <p>This method simply returns a new copy of the argument.
   *
   */
  private Object performSensorEffector(Object theTstObject) {
    if (theTstObject instanceof Boolean) {return new Boolean( ((Boolean)theTstObject).booleanValue() );}
    if (theTstObject instanceof Double ) {return new Double ( ((Double )theTstObject).doubleValue() ); }
    if (theTstObject instanceof String ) {return new String (  (String )theTstObject );                }

    return new Double(12.34);
  }


  //==================================================================
  //
  // Main method
  //
  //==================================================================
  public static void main(String[] args) {
    try {
      SampleSensorEffector lclSampleSensorEffector = new SampleSensorEffector();

      lclSampleSensorEffector.runSample();

      System.exit(0);
    }
    catch(Throwable theThrowable) {
      System.out.println("!! Main: caught exception: " + theThrowable);
      theThrowable.printStackTrace();
    }
  }

}
